/* This file is part of the KDE project

   Copyright (C) 2011 Ernesto Rodriguez <eortiz@uci,cu>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.
*/

#include "mmsthread.hpp"
#include "mmsTransfer.h"

#include "core/kget.h"

#include <kiconloader.h>
#include <klocale.h>
#include <kdebug.h>

#include <QFile>



MmsTransfer::MmsTransfer(TransferGroup * parent, TransferFactory * factory,
                         Scheduler * scheduler, const KUrl & source, const KUrl & dest,
                         const QDomElement * e)
    : Transfer(parent, factory, scheduler, source, dest, e)
{
    kDebug(5001) << "Mms transfer initialized: " + m_source.prettyUrl();
    
}

void MmsTransfer::init()
{
    this->dmms = mmsx_connect(NULL, NULL, qstrdup(m_source.prettyUrl().toAscii()), 1e9);
    slotTotalSize(mmsx_get_length(this->dmms));
    this->time.start();
}


void MmsTransfer::start()
{
    if (mmsx_get_seekable(this->dmms)){
      
      this->cnthds = 40;
      qDebug() << "Sekeable: " << mmsx_get_seekable(this->dmms) << "-- Hilos: " << this->listaHilos.length();
	
	int part = mmsx_get_length(this->dmms) / this->cnthds;
	int sum = 0;
	int count = 0;
	this->time.restart();
	for (int i = 0; i < this->cnthds; i++) {
	    if (i+1 == this->cnthds) part = m_totalSize - sum;
	    Mmsthread* hilo = new Mmsthread(m_source.prettyUrl(), m_dest.pathOrUrl(), sum, sum+part);
	    sum +=part; count++;
	    this->listaHilos.append(hilo);
	    connect(hilo, SIGNAL(finished()), this, SLOT(slotResult()));
	    connect(hilo, SIGNAL(reading(int)), this, SLOT(slotProcessedSize(int)));
	    hilo->start();
	    sleep(0.5);
	    if (hilo->isRunning()) qDebug() << "Running" << hilo->end << "--" << count;
	}
    }
    else{
      this->cnthds = 1;
      qDebug() << "Sekeable: " << mmsx_get_seekable(this->dmms) << "-- Hilos: " << this->listaHilos.length();
      
      Mmsthread* hilo = new Mmsthread(m_source.prettyUrl(), m_dest.pathOrUrl(), 0, mmsx_get_length(this->dmms));
      this->listaHilos.append(hilo);
      connect(hilo, SIGNAL(finished()), this, SLOT(slotResult()));
      connect(hilo, SIGNAL(reading(int)), this, SLOT(slotProcessedSize(int)));
      this->time.restart();
      hilo->start();
      
    }
      
    kDebug(5001) << "Trying to start Mms-transfer: " + m_source.prettyUrl();
    setStatus(Job::Running, i18nc("transfer state: running", "Running...."), SmallIcon("network-connect")); // should be "network-connecting", but that doesn't exist for KDE 4.0 yet
    setTransferChange(Tc_Status, true);
}

void MmsTransfer::stop()
{
    if ((status() == Stopped) || (status() == Finished)) {
        return;
    }
    
    foreach(Mmsthread* h, this->listaHilos) {
	h->stop = true;
    }
    this->listaHilos.clear();
    setStatus(Job::Stopped, i18nc("transfer state: stopped", "Stopped"), SmallIcon("process-stop"));
    m_downloadedSize = 0;
    m_percent = 0;
    m_downloadSpeed = 0;
    Transfer::ChangesFlags flags = Tc_Status;
    flags |= Tc_Percent | Tc_DownloadedSize | Tc_DownloadSpeed;
    setTransferChange(flags, true);
}

void MmsTransfer::deinit()
{
    if (status() != Job::Finished)//if the transfer is not finished, we delete the *.part-file
    {
        mmsx_close(this->dmms);
    }//TODO: Ask the user if he/she wants to delete the *.part-file? To discuss (boom1992)
}

void MmsTransfer::slotResult()
{   
    Transfer::ChangesFlags flags = Tc_Status;
    
    if (this->cnthds != 1){
	this->cnthds--;
    }
    else{
      qDebug() << m_downloadedSize << " ----  "<< m_totalSize;
	if (m_downloadedSize == m_totalSize){
	    setStatus(Job::Finished, i18nc("Transfer State: Finished", "Finished"), SmallIcon("dialog-ok"));
	    m_percent = 100;
	    m_downloadSpeed = 0;
	}
	else{
	    setStatus(Job::Stopped, i18nc("transfer state: stopped", "Stopped"), SmallIcon("process-stop"));
	    m_downloadedSize = 0; 
	    m_percent = 0;  
	    m_downloadSpeed = 0;
	}
	this->listaHilos.clear();
	flags |= Tc_Percent | Tc_DownloadedSize | Tc_DownloadSpeed;
    }
    setTransferChange(flags, true);
    /*foreach (Mmsthread* h, this->listaHilos){
	if (h->isFinished()){ 
	  delete(h);
	  this->listaHilos.removeAt(this->listaHilos.indexOf(h));
	}
    }*/
}		

void MmsTransfer::slotTotalSize(qulonglong size)
{
    m_totalSize = size;
    setTransferChange(Tc_TotalSize, true);
}

void MmsTransfer::slotProcessedSize(int size)
{   
    m_downloadedSize += size;
    setTransferChange(Tc_DownloadedSize, true);
    slotPercent();
    slotSpeed();
}

void MmsTransfer::slotPercent()
{   
    m_percent = (m_downloadedSize * 100)/m_totalSize;
    setTransferChange(Tc_Percent, true);
}

void MmsTransfer::slotSpeed()
{
    m_downloadSpeed = (m_downloadedSize*1000)/this->time.elapsed();
    setTransferChange(Tc_DownloadSpeed, true);
}
#include "mmsTransfer.moc"
